import React from 'react';
import { StyleSheet, View} from 'react-native';
import {
  Svg,
  Rect,
  Line,
  G,
  Polygon,
  Text as SvgText,
} from 'react-native-svg';

// get X corrdinate of a point of pentagon on radar chart.
const getXcoordinate = (x1, y1, portion) => {
  const cx = 125;
  const cy = 136;
   // distance function
  let d = Math.sqrt((x1 - cx) ** 2 + (y1 - cy) ** 2) * portion;
  // randian calculation
  let radian = Math.atan(Math.abs((y1 - cy) / (x1 - cx)));
  // distance changed
  let tx =  Math.cos(radian) * d;
  // check the direction of polygon point relative to the center.
  return (x1 < cx) ? (cx - tx) : (cx + tx);
}

// get Y coordinate of a point of pentagon on radar chart
const getYcoordinate = (x1, y1, portion) => {
   // center point of pentagon
  const cx = 125;
  const cy = 136;
  // distance function * portion (e.g. 1/5 of that portion)
  let d = Math.sqrt((x1 - cx) ** 2 + (y1 - cy) ** 2) * portion;
  // calculate the radian of line beteen horizontal plane
  // and straight line connected between center point and pentagon point
  let radian = Math.atan(Math.abs((y1 - cy) / (x1 - cx)));
  let ty =  Math.sin(radian) * d;
  return (y1 < cy) ? (cy - ty) : (cy + ty);
}

// visualize the pentagon of student
const getStudentResult = (resultList, fillColor='purple') => {
  let coordinates = "";
  // 120 26,220 96,184 220,60 220,26 96
  let outerCoordinateList = [
    {x: 120, y:26},
    {x: 220, y:96},
    {x: 184, y:220},
    {x: 60, y:220},
    {x: 26, y:96},
  ]

  for (let i = 0; i < outerCoordinateList.length; i++) {
    coordinates += getXcoordinate(outerCoordinateList[i].x, outerCoordinateList[i].y, resultList[i] / 5).toString();
    if (i != outerCoordinateList.length - 1) {
      coordinates = coordinates + ' ' + getYcoordinate(outerCoordinateList[i].x, outerCoordinateList[i].y, resultList[i] / 5).toString() + ',';
    } else {
      coordinates = coordinates + ' ' + getYcoordinate(outerCoordinateList[i].x, outerCoordinateList[i].y, resultList[i] / 5).toString();
    }
  }
  return (    <Polygon
                points={coordinates}
                fill={fillColor}
                stroke="#ffffbe"
                fillOpacity="0.8"
                strokeWidth="1"
              />
          );
}

const getPentagon = (portion, color='#ffffbe') => {
  let coordinates = "";
  /**
   * This is the const coordinate of the outer most polygon
   */
  let outerCoordinateList = [
    {x: 120, y:26},
    {x: 220, y:96},
    {x: 184, y:220},
    {x: 60, y:220},
    {x: 26, y:96},
  ]

  for (let i = 0; i < outerCoordinateList.length; i++) {
    coordinates += getXcoordinate(outerCoordinateList[i].x, outerCoordinateList[i].y, portion).toString();
    if (i != outerCoordinateList.length - 1) {
      coordinates = coordinates + ' ' + getYcoordinate(outerCoordinateList[i].x, outerCoordinateList[i].y, portion).toString() + ',';
    } else {
      coordinates = coordinates + ' ' + getYcoordinate(outerCoordinateList[i].x, outerCoordinateList[i].y, portion).toString();
    }
  }
  console.log(coordinates);
  return (
              <Polygon
                points={coordinates}
                fill="transparent"
                stroke={color}
                strokeWidth="3"
              />
          );
}

// radarchart coordinate 220,96
// angle between = tan-1(40/95), length : 72.111026

// radarchart coordinate 120, 26
// angle between = tan-1()
export const RadarChart = ({ navigation, dataList, backGroundColor, resultColor, radarSize, textColor, numberColor}) =>{
    const [data, setData] = React.useState({});
    const [resultList, setResultList] = React.useState([5,2,2,3.15,5]);
    const [keyList, setKeyList] = React.useState([]);
    React.useEffect(() => {
       if (resultColor != null && typeof resultColor != 'string') throw 'Please Enter a string as a result color input';
       
       if (backGroundColor != null && typeof backGroundColor != 'string') throw 'Please Enter a String as a background color input';
       
       if (radarSize != null && isNaN(radarSize)) throw 'Please Enter a number as a radar size';

       let convertObj2Arr = (data) => {
           let res = [];
           let keys = [];
           for (let key in data) {
               
               if (isNaN(data[key])) { 
                   throw 'Please check your Object mapped values, they should be number';
               }

               if (data[key] > 5) {
                   throw 'Please check your Object mapped values, they should be less than or equal to 5';
               }

               res.push(data[key]);
               keys.push(key);
           }
           return [res, keys];
       }

        if (dataList === null || typeof dataList !== 'object') {
           throw 'Please check dataList input type, it should be non-null object type';
        }
        
        setData(dataList);
       
        try {
           var [res, keys] = convertObj2Arr(dataList);
        } catch (error) {
           throw error;
        }

        if (res.length != 5) {
           throw 'Please check your object input, it should be an object with 5 keys';
        }

        
        setResultList(res);
        setKeyList(keys);
  }, []);

  // [0]體能
  // [1] 腦力
  // [2] 積極
  // [3] 團隊
  // [4] 技能
  const styles = StyleSheet.create({
   backgroundContainer:{
     flex: 1,
     sectionContainer: {
       marginTop: 32,
       paddingHorizontal: 24,
     },
     sectionTitle: {
       fontSize: 24,
       fontWeight: '600',
     },
     sectionDescription: {
       marginTop: 8,
       fontSize: 18,
       fontWeight: '400',
     },
     highlight: {
       fontWeight: '700',
     },
   },
 });

 /**
  *    let outerCoordinateList = [
    {x: 120, y:26},
    {x: 220, y:96},
    {x: 184, y:220},
    {x: 60, y:220},
    {x: 26, y:96},
  ] 
   coordinate should be the centre point of the label
  */

 const getLabelCoordinate = (centerPoint, width) => {
   return centerPoint - (width) / 2;
 };
 if (keyList == null || keyList[0] == null) return;
  return(
    <View style={styles.backgroundContainer}>
        <View style={{flex:0.6, justifyContent:'center', alignItems: 'center', overflow:'hidden',minWidth:140, minHeight:140}}>
        <Svg height="600" width="600">
          <G transform={`translate(145,160)scale(${radarSize ? radarSize : 1.2})`}>
            <G transform="translate(0,0)">
              <Rect
                x={100}
                y="5"
                rx="10"
                width={keyList[0].length * 25}
                height="20"
                fill="#ffd85b"
                transform="translate(0,-20)"
                />
                <SvgText
                  fill="black"
                  fontSize="12"
                  x="124"
                  y="0"
                  textAnchor="middle"
                >
                  {keyList[0]}
                </SvgText>
                <SvgText
                  fill="white"
                  fontSize="12"
                  x="124"
                  y="20"
                  textAnchor="middle"
                >
                  {(Number.isInteger(resultList[0])) ? (resultList[0] + ".0") : (resultList[0].toString())}
                </SvgText>
              </G>
              <G transform="translate(215,65)">
                <Rect
                  rx="10"
                  width={50}
                  height="20"
                  fill="#ffd85b"
                  transform=""
                  />
                  <SvgText
                    fill="black"
                    transform="translate(25,15)"
                    fontSize="12"
                    textAnchor="middle"
                  >
                    {keyList[1]}
                  </SvgText>
                  <SvgText
                    transform="translate(25,35)"
                    fill="white"
                    fontSize="12"
                    textAnchor="middle"
                  >
                    {(Number.isInteger(resultList[1])) ? (resultList[1] + ".0") : (resultList[1].toString())}
                  </SvgText>
                </G>
                <G transform="translate(185,220)">
                  <Rect
                    rx="10"
                    width="50"
                    height="20"
                    fill="#ffd85b"
                    transform=""
                    />
                    <SvgText
                      fill="black"
                      transform="translate(25,15)"
                      fontSize="12"
                      textAnchor="middle"
                    >
                      {keyList[2]}
                    </SvgText>
                    <SvgText
                      transform="translate(25,35)"
                      fill="white"
                      fontSize="12"
                      textAnchor="middle"
                    >
                      {(Number.isInteger(resultList[2])) ? (resultList[2] + ".0") : (resultList[2].toString())}
                    </SvgText>
                  </G>
                  <G transform="translate(10,225)">
                    <Rect
                      rx="10"
                      width="50"
                      height="20"
                      fill="#ffd85b"
                      transform=""
                      />
                      <SvgText
                        fill="black"
                        transform="translate(25,15)"
                        fontSize="12"
                        textAnchor="middle"
                      >
                        {keyList[3]}
                      </SvgText>
                      <SvgText
                        transform="translate(25,35)"
                        fill="white"
                        fontSize="12"
                        textAnchor="middle"
                      >
                        {(Number.isInteger(resultList[3])) ? (resultList[3] + ".0") : (resultList[3].toString())}
                      </SvgText>
                    </G>
                    <G transform="translate(-20,65)">
                      <Rect
                        rx="10"
                        width="50"
                        height="20"
                        fill="#ffd85b"
                        transform=""
                        />
                        <SvgText
                          fill="black"
                          transform="translate(25,15)"
                          fontSize="12"
                          textAnchor="middle"
                        >
                            {keyList[4]}
                        </SvgText>
                        <SvgText
                          transform="translate(25,35)"
                          fill="white"
                          fontSize="12"
                          textAnchor="middle"
                        >
                          {(Number.isInteger(resultList[4])) ? (resultList[4] + ".0") : (resultList[4].toString())}
                        </SvgText>
                      </G>
            <Polygon
              points="120 26,220 96,184 220,60 220,26 96"
              fill= {backGroundColor ? backGroundColor : '#91d9d4'}
              stroke="#ffffbe"
              strokeWidth="3"
            />
              {getPentagon(0.2,'white')}
              {getPentagon(0.4)}
              {getPentagon(0.6,'white')}
              {getPentagon(0.8, 'white')}
              {getStudentResult(resultList, resultColor? resultColor : 'purple')}
            <Line x1="220" y1="96" x2="125" y2="136" stroke="#ffffbe"  strokeWidth="2" />
            <Line x1="120" y1="26" x2="125" y2="136" stroke="#ffffbe"  strokeWidth="2" />
            <Line x1="184" y1="220" x2="125" y2="136" stroke="#ffffbe"  strokeWidth="2" />
            <Line x1="60" y1="220" x2="125" y2="136" stroke="#ffffbe"  strokeWidth="2" />
            <Line x1="26" y1="96" x2="125" y2="136" stroke="#ffffbe"  strokeWidth="2" />
          </G>
        </Svg>
        </View>
    </View>
  )
}

module.exports = RadarChart